# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################

import sys
import os
import gzip
import time
import math
import subprocess

try:
	from cStringIO import StringIO
except ImportError:
	import StringIO

from urlgrabber.grabber import URLGrabber, URLGrabError

from clive.console import Console
from clive.parse import PageParser
from clive.error import CliveError

__all__ = ['Nomad']

class Nomad:
	def run(self, opts, args, say):
		self.opts = opts
		self.args = args
		self.say = say

		self.found_urls = [] # [(video_url, video_filename, video_size),]
		self.error_urls = [] # [(raw_url, errmsg),]
		self.dld_videos = [] # [(video_filename),]
		self.encoded_videos = [] # [(video_filename, encoded_video_filename),]

		raw_urls = self._get_raw_urls(args)
		self._check_raw_urls(raw_urls)

		if not opts.emit:
			self._dl()
			if opts.re_encode != 'off':
				self._re_encode()
			if opts.play != 'off':
				self._play()
		else:
			self._emit()

	def _emit(self):
		for (url, size_str, size_bytes, fn, reget, exists) in self.found_urls:
			self.say('video: "%s" "%s" "%s"\n' % (url, fn, size_str))

	def _dl(self):
		total_mb = 0
		total_count = 0
		total_skipped = 0

		for (url, size_str, size_bytes, fn, reget, exists) in self.found_urls:
			if not exists:
				total_count += 1
				total_mb += float(size_str.strip("MB"))
			else:
				total_skipped += 1

		self.say('queue: %d (total: %.3fMB), failed: %d, skipped: %d.' % (
			total_count, total_mb, len(self.error_urls), total_skipped))

		if len(self.found_urls) == 0:
			raise SystemExit('error: nothing to extract.')

		for (url, size_str, size_bytes, fn, reget, exists) in self.found_urls:
			try:
				if not exists:
					self._dl_video(url, fn, reget)

				self.dld_videos.append((url, fn))

			except URLGrabError, err:

				self.say(err.strerror, is_error=1)
				self.error_urls.append((url, err.strerror))
	
	def _re_encode(self):
		try:
			for (url, file) in self.dld_videos:

				if file.lower().endswith('.%s' % self.opts.re_encode):
					s = 're-encode: source (%s) is already %s. skipping.' % (
						os.path.basename(file), self.opts.re_encode)
					self.say(s)
					continue

				output_file = '%s.%s' % (file, self.opts.re_encode)
				output_file_stderr = '%s.stderr' % output_file

				self.say('re-encode: %s -> %s' % (
					os.path.basename(file), os.path.basename(output_file)))

				cmd = self.opts.ffmpeg
				
				if sys.platform != 'win32':
					cmd += '>/dev/null 2>"%s"' % output_file_stderr

				lookup = [ ('%i',file), ('%o',output_file) ]

				for (_id, repl) in lookup:
					cmd = cmd.replace(_id, '"%s"' % repl)

				rc = subprocess.call(cmd, shell=1)

				if rc != 0:
					self.say('note: ffmpeg returned an error, ' +
						'dumping stderr (%s).' % output_file_stderr)
				else:
					try:
						os.remove('%s' % output_file_stderr)
					except:
						pass
					self.encoded_videos.append((file, output_file))

		except OSError, e:
			self.say(str(e), is_error=1)
	
	def _play(self):
		play_videos = self.dld_videos

		if self.opts.play != 'src':
			play_videos = self.encoded_videos

		for (ignore, file) in play_videos:
			self.say('play: %s' % os.path.basename(file))

			lookup = [ ('%i',file) ]

			cmd = self.opts.player
			
			if sys.platform != 'win32':
				cmd += ' >/dev/null 2>&1'

			for (_id, repl) in lookup:
				cmd = cmd.replace(_id, '"%s"' % repl)
				
			rc = subprocess.call(cmd, shell=1)

			if rc != 0:
				self.say('note: player returned an error (%d).' % rc)

	def _get_raw_urls(self, args):
		if not args:
			c = Console(self.opts, self.say)
			(urls, self.opts) = c.read()
			return urls
		else:
			return args
	
	def _check_raw_urls(self, raw_urls):
		self.say('status: checking urls...') 

		for (index, url) in enumerate(raw_urls):
			self._check_url(url, (index,len(raw_urls)))
	
	def _check_url(self, url, index):
		if not url.startswith('http://'):
			url = 'http://' + url

		url = url.replace('/v/','/watch?v=') # ytube

		try:
			http_headers = ()
			if self.opts.accept_gzip:
				http_headers += ('accept-encoding', 'gzip'),

			g = URLGrabber(
				user_agent = self.opts.user_agent,
				http_headers = http_headers,
				progress_obj = ProgressVerify(index),
				throttle = self.opts.throttle,
				proxies = self._get_proxy()
			)

			o = g.urlopen(url)
			data = o.read()

			self.say("\n", show_always=1)

			if o.hdr.get('content-encoding') == 'gzip':
				data = gzip.GzipFile(fileobj=StringIO(data)).read()

			self.found_urls.append(self._parse_page(data, url))

		except URLGrabError, err:

			self.say(err.strerror, is_error=1)
			self.error_urls.append((url, err.strerror))

		except CliveError, err:

			self.say(err.errmsg, is_error=1)
			self.error_urls.append((url, err.errmsg))
	
	def _parse_page(self, data, url):
		parser = PageParser()
		return parser.parse(data, url, self.opts,
			self._callb_query_video_length, self.say)
	
	def _callb_query_video_length(self, video_url):
		self.say('status: checking file length... ', newline=0)

		g = URLGrabber(
			user_agent = self.opts.user_agent,
			throttle = self.opts.throttle,
			proxies = self._get_proxy()
		)

		o = g.urlopen(video_url)
		o.close()

		bytes = long(o.hdr.get('content-length'))
		mbytes = '%.3fMB' % (float(bytes) / (1024*1024))

		self.say('%s' % mbytes)

		return (mbytes, bytes)
	
	def _dl_video(self, url, file, reget, omit_say=0):
		if not omit_say:
			self.say('write: %s' % file)

		try:
			g = URLGrabber(
				user_agent = self.opts.user_agent,
				throttle = self.opts.throttle,
				progress_obj = Progress(),
				proxies = self._get_proxy(),
				reget = reget
			)

			file = g.urlgrab(url, filename=file)

		except URLGrabError, err:

			try:
				# 403: Fail-safe for resuming dl, overwrite file
				# if server returns 403 (forbidden)
				if getattr(err, 'code') == 403:
					if reget == 'simple':
						self.say('fail-safe: server forbids ' +
							'resuming, enforcing `--overwrite`')
						self.say('fail-safe: ctrl-c now if you '
							'do not want to continue; sleeping for 5s.')
						time.sleep(5)
						self._dl_video(url, file, reget=None, omit_say=1)

			except AttributeError:
				pass

			raise err # Unhandled, pass the error on

	def _get_proxy(self):
		# If self.opts.proxy == None:
		#	proxy was not set and http_proxy env. var. was not used,
		#	see also clive/opts.py which checks for the http_proxy
		#
		# NOTE: Returning 'None' from this method disables the use
		#	of proxy altogether.
		proxy = self.opts.proxy
		if not proxy:
			return None
		else:
			if not proxy.startswith('http://'):
				proxy = 'http://' + proxy
		return {'http':proxy}


class Progress:

	def start(self, filename, url, basename, length, text):
		self.length = length
		self.ttime = time.time()
		self.filename = filename
		self.url = url
	
	def update(self, read):
		if read == 0: return

		progress = self._percent(read, self.length)

		if progress > 0:
			text = 'transfer: %s / %s (%03d%%)' % (
				self._bytesmb(read),
				self._bytesmb(self.length),
				progress
			)

			elapsed = time.time() - self.ttime
			if elapsed == 0: elapsed=1

			rate = read / elapsed
			left = (self.length - read) / rate
			gmt = time.gmtime(left)

			left = '%02d:%02d:%02d' % (gmt[3], gmt[4], gmt[5])
			text += ' [%s] %12s/s' % (left, self._byteshuman(rate))
		else:
			text = 'transfer: %s' % self._bytesmb(read)

		self._write_bspaced(text)

	def end(self, total):
		self.total = total
		self._write_stdout('\n')
	
	def _percent(self, i, j):
		if not i or not j or i == 0 or j == 0: return 0
		return int(float(i) / float(j) * 100)
	
	def _byteskb(self, bytes):
		return '%.0fKB' % (float(bytes) / 1024)

	def _bytesmb(self, bytes):
		return '%.3fMB' % (float(bytes) / (1024*1024))
	
	def _byteshuman(self, bytes):
		a = ['bytes','KB','MB','GB','TB']
		try:
			i = int(math.floor(math.log(bytes,1024)))
			b = round(float(bytes/math.pow(1024,i)),2)
		except OverflowError:
			i = 0
			b = 0.0
		return '%.2f%s' % (b,a[i])

	def _write_bspaced(self, text):
		bspaces = '\b' * len(text)
		self._write_stdout(bspaces + text)
	
	def _write_stdout(self, text):
		sys.stdout.write(text)
		sys.stdout.flush()


class ProgressVerify(Progress):

	def __init__(self, index):
		self.index = index

	def update(self, read):
		if read == 0: return

		progress = self._percent(read, self.length)

		if progress > 0:
			text = 'status: %s %03d%% (%d of %d) [%s]' % (
				self._byteskb(read), progress,
				self.index[0]+1, self.index[1], self.url)
		else:
			text = 'status: %s (%d of %d) [%s]' % (
				self._byteskb(read), self.index[0]+1,
				self.index[1], self.url)

		text = text[:70]
		if len(text) == 70:
			text += '...]'

		self._write_bspaced(text)


