# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################

import os
import sys

from optparse import OptionParser, OptionGroup

import urlgrabber as _urlgrabber

import clive as _clive
from clive.nomad import Nomad

try:
	import win32clipboard
	import win32con
	_mswin_clipboard_avail = 1
except ImportError:
	_mswin_clipboard_avail = 0

__all__ = ['Options']

class Options:
	_rcfile = 'config.py'
	_rcdir = '.clive'

	def __init__(self):
		if sys.platform == 'win32':
			self._rcdir = self._rcdir.strip('.')
		path = os.path.join(os.path.expanduser('~'), self._rcdir)
		self.rcfile = os.path.join(path, self._rcfile)
		self.warnings = []
	
	def parse(self):
		self._parseopts()
		return (self.opts, self.args)

	def show(self, say):
		say('%s (python-%s/urlgrabber-%s)' % (
			_clive.__version_str__,
			sys.version.split()[0],
			_urlgrabber.__version__)
		)
		yesoff = ['off','yes']
		say('kb/s:%3s  gzip:%s  agent:%s' % (
			(['off', '%.0f' % (self.opts.throttle/1024)] \
			[self.opts.throttle > 0]),
			(yesoff[self.opts.accept_gzip]),
			self.opts.user_agent)
		)
		say('o/w :%s  r/n :%s  proxy:%s' % (
			(yesoff[self.opts.overwrite]),
			(yesoff[self.opts.rename]),
			([self.opts.proxy,'off'][not self.opts.proxy]))
		)

		say('play:%s  r/en:%s  ffmpg:%s' % (
			self.opts.play, self.opts.re_encode,
			[self.opts.ffmpeg,'off'][not self.opts.ffmpeg]))

		for w in self.warnings:
			say(w)

	# Non-public

	def _parseopts(self):
		p = OptionParser(
			usage = '%prog [options] url',
			description = _clive.__doc__,
			version = _clive.__version_str__+ '\n' + _clive.__notice__
		)

		gstdout = OptionGroup(p, 'Stdout Options', None)
		goutput = OptionGroup(p, 'Output Options', None)
		gclip = OptionGroup(p, 'Clipboard options', None)
		gconn = OptionGroup(p, 'Connection Options', None)
		gplayback = OptionGroup(p, 'Playback Options', None)
		gencode = OptionGroup(p, 'Re-encode Options', None)

		p.add_option_group(gstdout)
		p.add_option_group(goutput)
		p.add_option_group(gclip)
		p.add_option_group(gconn)
		p.add_option_group(gplayback)
		p.add_option_group(gencode)

		lookup = [
			(gstdout, 'verbose', '--quiet', '-q', 'store_false', None, None,
				'Run quiet, kill verbosity', None, None),
			(gstdout, 'emit', '--emit', '-e', 'store_true', None, None,
				'Show a list of found video extraction urls and exit',
				None, None),

			(goutput, 'overwrite', '--overwrite', '-o', 'store_true', None,
				None, 'Overwrite already existing output files', None, None),
			(goutput, 'rename', '--rename', '-r', 'store_true', None, None,
				'Generate a unique output file name if file exists ' +
				'already', None, None),
			(goutput, 'prefix', '--prefix', None, 'store', None, None,
				'Write extracted video files to PREFIX', None, None),
			(goutput, 'filename_chars', '--filename-chars', None, 'store',
				None, None, 'Set FILENAME_CHARS as new regex mask for output' +
				'filenames', None, None),
			(goutput, 'filename_format', '--filename-format', None, 'store',
				None, None, 'Set FILENAME_FORMAT as new output filename format',
				None, None),

			(gclip, 'xclip', '--xclip', None, 'store', None, None,
				'Set XCLIP as path to xclip, e.g. --xclip="/usr/bin/xclip -o"',
				None, None),
			(gclip, 'from_clipboard', '--from-clipboard', '-c', 'store_true',
				None, None,
				'Read URLs from the clipboard (unix/x: req. --xclip, ' +
				'mswin: req. win32clipboard module)',
				None, None),

			(gconn, 'proxy', '--proxy', None, 'store', None, None,
				'Set PROXY as HTTP proxy for all connections',	None, None),
			(gconn, 'user_agent', '--user-agent', None, 'store', None, None,
				'Identify clive as USER_AGENT for all connections', None, None),
			(gconn, 'throttle', '--throttle', None, 'store', None, None,
				'Set THROTTLE as new limit (KB/s) for all HTTP transfers',
				None, 'int'),
			(gconn, 'accept_gzip', '--disable-gzip', None, 'store_false',
				None, None, 'Disable gzip encoding for (video) page ' +
				'transfers explicitly', None, None),

			(gplayback, 'player', '--player', None, 'store', None, None,
				'Set PLAYER as path to a player command', None, None),
			(gplayback, 'play', '--play', '-p', 'store',
				None, ['src','s','mpg','m','avi','a'],
				'Play PLAY format ("src", "mpg", "avi")',
				None, None),
			(gplayback, 'play', '--disable-play', None, 'store_const',
				'off', None, 'Disable playback explicitly', None, None),

			(gencode, 'ffmpeg', '--ffmpeg', None, 'store', None, None,
				'Set FFMPEG as new path to the ffmpeg command', None, None),
			(gencode, 're_encode', '--re-encode', '-R', 'store',
				None, ['mpg','m', 'avi','a'],
				'Re-encode extracted video to RE_ENCODE video format ("mpg, "avi")',
				None, None),
			(gencode, 're_encode', '--disable-encode', None, 'store_const',
				'off', None, 'Disable re-encoding explicitly', None, None),
		]

		for (group, dest, lname, sname, action, const, choices, desc, \
			callback, _type) in lookup:
			group.add_option(
				lname, sname,
				dest=dest,
				action=action,
				const=const,
				choices=choices,
				callback=callback,
				type=_type,
				help=desc
			)

		try:
			self.config = eval(open(self.rcfile).read())
		except IOError:
			self.config = None

		p.set_defaults(verbose = self._getconf('verbose', 1, 1))
		p.set_defaults(emit=0)

		p.set_defaults(overwrite = self._getconf('overwrite', 0, 1))
		p.set_defaults(rename = self._getconf('rename', 0, 1))
		p.set_defaults(prefix = self._getconf('prefix', os.getcwd()))
		p.set_defaults(filename_chars =
			self._getconf('filename_chars', 'A-Za-z0-9'))
		p.set_defaults(filename_format =
			self._getconf('filename_format', '%t.%e'))

		p.set_defaults(xclip = self._getconf('xclip'))
		p.set_defaults(from_clipboard =
			self._getconf('from_clipboard', 0, 1))

		p.set_defaults(proxy = self._getconf('proxy'))
		p.set_defaults(user_agent =
			self._getconf('user_agent', 'clive/'+_clive.__version__))
		p.set_defaults(throttle = self._getconf('throttle', 0))
		p.set_defaults(accept_gzip = self._getconf('accept_gzip', 1))

		p.set_defaults(player = self._getconf('player'))
		p.set_defaults(play = self._getconf('play',
			choices=['src','s','mpg','m','avi','a']))

		p.set_defaults(ffmpeg = self._getconf('ffmpeg'))
		p.set_defaults(re_encode = self._getconf('re_encode',
			choices=['mpg','m','avi','a']))

		(self.opts, self.args) = p.parse_args()

		# Exceptions / tweaks

		proxy = os.environ.get('http_proxy')
		if proxy and not self.opts.proxy:
			self.opts.proxy = proxy

		lookup = {
			None:'off',
			's':'src',
			'm':'mpg',
			'a':'avi'
		}

		for l in lookup:
			if self.opts.re_encode == l:
				self.opts.re_encode = lookup[l]
			if self.opts.play == l:
				self.opts.play = lookup[l]

		if not os.path.exists(self.opts.prefix):
			self.opts.prefix = os.getcwd()
			self.warnings.append('warn: prefix path does not exist; ' +
				'using current working dir instead')

		self.opts.throttle *= 1024 # kb -> bytes (urlgrabber)

		# NOTE: Mighty messy.

		if self.opts.player: # Ensure --player was supplied with '%i'
			if self.opts.player.find('%i') == -1 and \
				self.opts.play != 'off':
				self.warnings.append('warn: --player does not ' +
					'specify %i, playback disabled.')
				if self.opts.play != 'src':
					self.warnings.append('warn: re-encoding disabled due ' +
					'to the above.')
					self.opts.re_encode = 'off'
				self.opts.play = 'off'

		if self.opts.ffmpeg: # Ensure --ffmpeg was supplied with '%i' and '%o'
			lookup = ['%i','%o']
			for l in lookup:
				if self.opts.ffmpeg.find(l) == -1 and \
				self.opts.re_encode != 'off':
					self.warnings.append('warn: --ffmpeg does not ' +
						'specify %s, re-encoding disabled.' % l)
					self.opts.re_encode = 'off'
					if self.opts.play != 'off' and self.opts.play != 'src':
						self.opts.re_encode = 'off'

		if self.opts.play != 'off' or self.opts.re_encode != 'off':

			if not self.opts.player and self.opts.play != 'off':
				# Make sure --player was set if --play is being used
				self.warnings.append('warn: --player not set, ' +
					'playback disabled.')
				self.opts.play = 'off'

			if not self.opts.ffmpeg and (self.opts.play != 'src' or \
				self.opts.re_encode != 'off'):
				# Make sure --ffmpeg was set if --play is != 'src' 
				self.warnings.append(
					'warn: --ffmpeg not set, playback and re-encoding ' +
					'disabled.')
				self.opts.play = 'off'
				self.opts.re_encode = 'off'

			else: # Set --re-encode as --play if we get this far
				if self.opts.play != 'off' and self.opts.play != 'src':
					self.opts.re_encode = self.opts.play

		if self.opts.from_clipboard: # Check clipboard options
			if sys.platform != 'win32':
				if not self.opts.xclip:
					self.warnings.append('warn: --xclip not used, ' +
						'disabling --from-clipboard')
					self.opts.from_clipboard = 0
			else:
				if not _mswin_clipboard_avail:
					self.warnings.append('warn: win32clipboard module ' +
						'not found, disabling --from-clipboard')
					self.opts.from_clipboard = 0


	def _getconf(self, key, default=None, return_bool=0, choices=None):
		if not self.config:
			return default

		try:
			value = self.config[key]

			if return_bool:
				value = self._istrue(value)

			elif choices:
				try:
					choices.index(value)
				except ValueError:
					raise SystemExit('%s:%s: invalid choice: "%s", ' \
						'choose from %s' % (self.rcfile, key,
							value, repr(choices)))
			return value

		except KeyError:
			return default

	def _istrue(self, value):
		if type(value) == str:
			v = value.lower()
			return (v == 'on' or v == 'yes' or v == '1' or v == 'true')
		else:
			return value == 1


