# -*- coding: ascii -*-

###########################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 0.1.2-1307 USA
###########################################################################

import os
import urlparse
import urllib
import formatter
import time
import md5
import re

from htmllib import HTMLParser

from clive.error import CliveError

__all__ = ['PageParser']

class PageParser:
	def parse(self, data, url, opts, callb_query_video_length, say):
		self.say = say

		fmt = formatter.AbstractFormatter(formatter.NullWriter())
		p = HTMLParser(fmt)
		p.feed(data)
		p.close()

		if url.find('youtube.com') != -1:
			video_url = self._parse_ytube(data)
		elif url.find('video.google.') != -1:
			video_url = self._parse_vgoogle(data)
		elif url.find('dailymotion.com') != -1:
			video_url = self._parse_dmotion(data)
		elif url.find('guba.com') != -1:
			video_url = self._parse_guba(data)
		elif url.find('stage6.divx.com') != -1:
			video_url = self._parse_stage6(data)
		else:
			(scheme, host, path, params, query, fragment) = \
				urlparse.urlparse(url)
			raise CliveError('error: unknown host (%s)' % host)

		if len(video_url) == 0:
			raise CliveError('error: extraction url not found')

		video_details = (video_url,)
		video_details += callb_query_video_length(video_url)
		video_details += self._get_video_filename(
			p.title, url, opts, video_details
		)

		# (video_url, video_length_str, video_length_bytes, 
		# 	video_filename, reget)
		return video_details

	def _parse_ytube(self, data):
		ldata = data.lower()

		if ldata.find('please verify you are 18') != -1:
			raise CliveError('error: age verification')
		elif ldata.find('no longer available') != -1 or \
			ldata.find('has been removed') != -1 or \
			ldata.find('this video is unavailable') != -1:
			raise CliveError('error: video is unavailable')
		elif ldata.find('url contained a malformed video id') != -1:
			raise CliveError('error: url contained a malformed video id')
	
		try:
			video_id = \
				self._parse_from_to(data, "video_id:'", ",", skip_from=1)
			video_id = video_id.replace("'", "")
			if len(video_id) == 0:
				raise CliveError()
		except:
			raise CliveError('error: extraction url not found')
		
		try:
			ticket = self._parse_from_to(data, "t:'", ",", skip_from=1)
			ticket = ticket.replace("'", "")
			if len(ticket) == 0: raise CliveError()
		except:
			raise CliveError('error: ticket not found in page html')

		return 'http://www.youtube.com/get_video?video_id=%s&t=%s' % (
			video_id, ticket
		)

	def _parse_vgoogle(self, data):
		text = self._parse_from_to(data, 'googleplayer.swf', '"')
		try:
			return urllib.unquote(text.split('=',1)[1])
		except IndexError:
			pass
		return ''
	
	def _parse_dmotion(self, data):
		text = self._parse_from_to(data, 'url=', '"')

		if len(text) == 0:
			ldata = data.lower()
			if ldata.find('content deleted.') != -1:
				raise CliveError('error: video has been removed')
			elif ldata.find('explicit content.') != -1:
				raise CliveError('error: inappropriate content')
			else:
				raise CliveError('error: extraction url not found')

		try:
			return urllib.unquote(text.split('url=',2)[2])
		except IndexError:
			pass
		return ''
	
	def _parse_guba(self, data):
		return self._parse_from_to(data,
			'http://free.guba.com/uploaditem/', '"')
	
	def _parse_stage6(self, data):
		return self._parse_from_to(data,
			'http://video.stage6.com/', '&')

	def _parse_from_to(self, data, _from, to, skip_from=0):
		start = data.find(_from)
		end = data.find(to, start)
		text = ''

		if start != -1 and end != -1:
			if skip_from: start += len(_from)
			text = data[start:end]

		return text
	
	def _get_video_filename(self, title, url, opts, video_details):
		reget = None
		exists = 0
		(video_url, video_length_str, video_length_bytes) = video_details

		#ext = (['flv','avi'][url.find('video.google.') != -1])
		ext = 'flv'
		title = title.replace('YouTube -', '')
		title = title.replace('GUBA -', '')

		try:
			if title.lower().find('- dailymotion') != -1:
				title = title.split(' - ', 1)[0]
				title = title.split('Video ', 1)[1]

			if title.lower().find('stage6 ') != -1:
				title = title.split(' ', 2)[2]
				title = title.split('-', 1)[0]

		except IndexError:
			pass

		title = re.sub('[^%s]' % opts.filename_chars, '', title)

		if len(title) == 0:
			title = self._random_string(insert_dash=0)

		if len(title) > 64:
			title = title[:64]

		host = vid = ''

		if url.lower().find('youtube.com') != -1:
			host = 'ytube'
			try:
				vid = url.split('watch?v=',1)[1].split('&',1)[0]
			except IndexError:
				vid = md5.new(str(time.time())).hexdigest()[:8]
		elif url.lower().find('video.google.') != -1:
			host = 'vgoogle'
			try:
				vid = url.split('docid=',1)[1].split('&',1)[0]
			except IndexError:
				vid = md5.new(str(time.time())).hexdigest()[:8]
		elif url.lower().find('dailymotion.com') != -1:
			host = 'dmotion'
			try:
				vid = url.rsplit('/',1)[1].split('_',1)[0]
			except IndexError:
				vid = md5.new(str(time.time())).hexdigest()[:8]
		elif url.lower().find('guba.com') != -1:
			host = 'guba'
			try:
				vid = url.split('watch/',1)[1].split('?',1)[0]
			except IndexError:
				vid = md5.new(str(time.time())).hexdigest()[:8]
		elif url.lower().find('stage6.divx.com') != -1:
			host = 'stage6'
			ext = 'avi' # !
			try:
				vid = url.split('/video/',1)[1].split('/')[0]
			except IndexError:
				vid = md5.new(str(time.time())).hexdigest()[:8]

		lookup = [ ('%t',title), ('%i',vid), ('%h',host), ('%e',ext) ]
		filename = opts.filename_format

		for (_id, repl) in lookup:
			filename = filename.replace(_id, repl)

		if opts.prefix:
			filename = os.path.join(opts.prefix, filename)

		if os.path.exists(filename):
			if os.path.getsize(filename) < video_length_bytes:
				if host == 'dmotion' or host == 'guba' or host == 'stage6':
					reget = 'simple' # Try to resume dl
				else:
					# Enforce --rename for all others
					if not opts.overwrite:
						self.say('note: resuming not supported by host ' +
							'enforcing `--rename`')
						filename = self._rename_file(filename)
			else:
				if not opts.overwrite:
					if opts.rename:
						filename = self._rename_file(filename)
					else:
						exists = 1
						self.say('warn: %s of same size exists already; ' \
						'will not dl video.' % filename)
				else:
					pass # Overwrite existing file

		return (filename, reget, exists)
	
	def _rename_file(self, filename):
		(root, ext) = os.path.splitext(filename)
		return root + self._random_string() + ext

	def _random_string(self, insert_dash=1):
		s = ''
		if insert_dash: s = '-'
		s += md5.new(str(time.time())).hexdigest()[:8]
		s += time.strftime('-%Y-%m-%dT%H:%M:%S').replace(':', '_')
		return s


